@@ -65,14 +65,27 @@ struct ena_hw_metrics {
 }
 
 static const struct ena_stats ena_stats_global_strings[] = {
+	ENA_STAT_GLOBAL_ENTRY(total_resets),
+	ENA_STAT_GLOBAL_ENTRY(reset_fail),
 	ENA_STAT_GLOBAL_ENTRY(tx_timeout),
+	ENA_STAT_GLOBAL_ENTRY(wd_expired),
+	ENA_STAT_GLOBAL_ENTRY(admin_q_pause),
+	ENA_STAT_GLOBAL_ENTRY(bad_tx_req_id),
+	ENA_STAT_GLOBAL_ENTRY(bad_rx_req_id),
+	ENA_STAT_GLOBAL_ENTRY(bad_rx_desc_num),
+	ENA_STAT_GLOBAL_ENTRY(missing_intr),
+	ENA_STAT_GLOBAL_ENTRY(suspected_poll_starvation),
+	ENA_STAT_GLOBAL_ENTRY(missing_tx_cmpl),
+	ENA_STAT_GLOBAL_ENTRY(rx_desc_malformed),
+	ENA_STAT_GLOBAL_ENTRY(tx_desc_malformed),
+	ENA_STAT_GLOBAL_ENTRY(invalid_state),
+	ENA_STAT_GLOBAL_ENTRY(os_netdev_wd),
+	ENA_STAT_GLOBAL_ENTRY(missing_admin_interrupt),
+	ENA_STAT_GLOBAL_ENTRY(admin_to),
 	ENA_STAT_GLOBAL_ENTRY(suspend),
 	ENA_STAT_GLOBAL_ENTRY(resume),
-	ENA_STAT_GLOBAL_ENTRY(wd_expired),
-	ENA_STAT_GLOBAL_ENTRY(interface_up),
 	ENA_STAT_GLOBAL_ENTRY(interface_down),
-	ENA_STAT_GLOBAL_ENTRY(admin_q_pause),
-	ENA_STAT_GLOBAL_ENTRY(reset_fail),
+	ENA_STAT_GLOBAL_ENTRY(interface_up),
 };
 
 /* A partial list of hw stats. Used when admin command
@@ -719,19 +732,19 @@ static void ena_get_drvinfo(struct net_device *dev,
 
 	ret = strscpy(info->driver, DRV_MODULE_NAME, sizeof(info->driver));
 	if (ret < 0)
-		netif_info(adapter, drv, dev,
-			   "module name will be truncated, status = %zd\n", ret);
+		netif_dbg(adapter, drv, dev,
+			  "module name will be truncated, status = %zd\n", ret);
 
 	ret = strscpy(info->version, DRV_MODULE_GENERATION, sizeof(info->version));
 	if (ret < 0)
-		netif_info(adapter, drv, dev,
-			   "module version will be truncated, status = %zd\n", ret);
+		netif_dbg(adapter, drv, dev,
+			  "module version will be truncated, status = %zd\n", ret);
 
 	ret = strscpy(info->bus_info, pci_name(adapter->pdev),
 		sizeof(info->bus_info));
 	if (ret < 0)
-		netif_info(adapter, drv, dev,
-			   "bus info will be truncated, status = %zd\n", ret);
+		netif_dbg(adapter, drv, dev,
+			  "bus info will be truncated, status = %zd\n", ret);
 
 	info->n_priv_flags = ENA_PRIV_FLAGS_NR;
 }
@@ -749,6 +762,23 @@ static void ena_get_ringparam(struct net_device *netdev,
 
 	ring->tx_max_pending = adapter->max_tx_ring_size;
 	ring->rx_max_pending = adapter->max_rx_ring_size;
+#ifdef ENA_LARGE_LLQ_ETHTOOL
+	if (adapter->ena_dev->tx_mem_queue_type == ENA_ADMIN_PLACEMENT_POLICY_DEV) {
+		bool large_llq_supported = adapter->large_llq_header_supported;
+
+		kernel_ring->tx_push = true;
+		kernel_ring->tx_push_buf_len = adapter->ena_dev->tx_max_header_size;
+		if (large_llq_supported)
+			kernel_ring->tx_push_buf_max_len = ENA_LLQ_LARGE_HEADER;
+		else
+			kernel_ring->tx_push_buf_max_len = ENA_LLQ_HEADER;
+	} else {
+		kernel_ring->tx_push = false;
+		kernel_ring->tx_push_buf_max_len = 0;
+		kernel_ring->tx_push_buf_len = 0;
+	}
+
+#endif
 	ring->tx_pending = adapter->tx_ring[0].ring_size;
 	ring->rx_pending = adapter->rx_ring[0].ring_size;
 }
@@ -763,7 +793,8 @@ static int ena_set_ringparam(struct net_device *netdev,
 #endif
 {
 	struct ena_adapter *adapter = netdev_priv(netdev);
-	u32 new_tx_size, new_rx_size;
+	u32 new_tx_size, new_rx_size, new_tx_push_buf_len;
+	bool changed = false;
 
 	if (ring->rx_mini_pending || ring->rx_jumbo_pending)
 		return -EINVAL;
@@ -776,11 +807,53 @@ static int ena_set_ringparam(struct net_device *netdev,
 				adapter->max_rx_ring_size);
 	new_rx_size = rounddown_pow_of_two(new_rx_size);
 
-	if (new_tx_size == adapter->requested_tx_ring_size &&
-	    new_rx_size == adapter->requested_rx_ring_size)
+	changed |= new_tx_size != adapter->requested_tx_ring_size ||
+		   new_rx_size != adapter->requested_rx_ring_size;
+
+	/* This value is ignored if LLQ is not supported */
+	new_tx_push_buf_len = adapter->ena_dev->tx_max_header_size;
+#ifdef ENA_LARGE_LLQ_ETHTOOL
+
+	if ((adapter->ena_dev->tx_mem_queue_type == ENA_ADMIN_PLACEMENT_POLICY_DEV) !=
+	    kernel_ring->tx_push) {
+		NL_SET_ERR_MSG_MOD(extack, "Push mode state cannot be modified");
+		return -EINVAL;
+	}
+
+	/* Validate that the push buffer is supported on the underlying device */
+	if (kernel_ring->tx_push_buf_len) {
+		enum ena_admin_placement_policy_type placement;
+
+		new_tx_push_buf_len = kernel_ring->tx_push_buf_len;
+
+		placement = adapter->ena_dev->tx_mem_queue_type;
+		if (placement == ENA_ADMIN_PLACEMENT_POLICY_HOST)
+			return -EOPNOTSUPP;
+
+		if (new_tx_push_buf_len != ENA_LLQ_HEADER &&
+		    new_tx_push_buf_len != ENA_LLQ_LARGE_HEADER) {
+			bool large_llq_sup = adapter->large_llq_header_supported;
+			char large_llq_size_str[40];
+
+			snprintf(large_llq_size_str, 40, ", %lu", ENA_LLQ_LARGE_HEADER);
+
+			NL_SET_ERR_MSG_FMT_MOD(extack,
+					       "Supported tx push buff values: [%lu%s]",
+					       ENA_LLQ_HEADER,
+					       large_llq_sup ? large_llq_size_str : "");
+
+			return -EINVAL;
+		}
+
+		changed |= new_tx_push_buf_len != adapter->ena_dev->tx_max_header_size;
+	}
+
+#endif
+	if (!changed)
 		return 0;
 
-	return ena_update_queue_sizes(adapter, new_tx_size, new_rx_size);
+	return ena_update_queue_params(adapter, new_tx_size, new_rx_size,
+				       new_tx_push_buf_len);
 }
 
 #ifdef ETHTOOL_GRXRINGS
@@ -871,7 +944,7 @@ static int ena_get_rss_hash(struct ena_com_dev *ena_dev,
 	}
 
 	rc = ena_com_get_hash_ctrl(ena_dev, proto, &hash_fields);
-	if (rc)
+	if (unlikely(rc))
 		return rc;
 
 	cmd->data = ena_flow_hash_to_flow_type(hash_fields);
@@ -1023,7 +1096,7 @@ static int ena_indirection_table_get(struct ena_adapter *adapter, u32 *indir)
 		return 0;
 
 	rc = ena_com_indirect_table_get(ena_dev, indir);
-	if (rc)
+	if (unlikely(rc))
 		return rc;
 
 	/* Our internal representation of the indices is: even indices
@@ -1046,7 +1119,7 @@ static int ena_get_rxfh(struct net_device *netdev, u32 *indir, u8 *key,
 	int rc;
 
 	rc = ena_indirection_table_get(adapter, indir);
-	if (rc)
+	if (unlikely(rc))
 		return rc;
 
 	/* We call this function in order to check if the device
@@ -1090,7 +1163,7 @@ static int ena_get_rxfh(struct net_device *netdev, u32 *indir, u8 *key)
 	int rc;
 
 	rc = ena_indirection_table_get(adapter, indir);
-	if (rc)
+	if (unlikely(rc))
 		return rc;
 
 	/* We call this function in order to check if the device
@@ -1202,15 +1275,20 @@ static int ena_set_channels(struct net_device *netdev,
 	struct ena_adapter *adapter = netdev_priv(netdev);
 	u32 count = channels->combined_count;
 	/* The check for max value is already done in ethtool */
-#ifdef ENA_XDP_SUPPORT
-	if (count < ENA_MIN_NUM_IO_QUEUES ||
-	    (ena_xdp_present(adapter) &&
-	    !ena_xdp_legal_queue_count(adapter, count)))
-#else
 	if (count < ENA_MIN_NUM_IO_QUEUES)
-#endif /* ENA_XDP_SUPPORT */
 		return -EINVAL;
 
+	if (!ena_xdp_legal_queue_count(adapter, count)) {
+		if (ena_xdp_present(adapter))
+			return -EINVAL;
+
+		xdp_clear_features_flag(netdev);
+	} else {
+		xdp_set_features_flag(netdev,
+				      NETDEV_XDP_ACT_BASIC |
+				      NETDEV_XDP_ACT_REDIRECT);
+	}
+
 	if (count > adapter->max_num_io_queues)
 		return -EINVAL;
 	if (count != adapter->num_io_queues && ena_is_zc_q_exist(adapter)) {
@@ -1289,6 +1367,10 @@ static const struct ethtool_ops ena_ethtool_ops = {
 	.supported_coalesce_params = ETHTOOL_COALESCE_USECS |
 				     ETHTOOL_COALESCE_USE_ADAPTIVE_RX,
 #endif
+#ifdef ENA_LARGE_LLQ_ETHTOOL
+	.supported_ring_params	= ETHTOOL_RING_USE_TX_PUSH_BUF_LEN |
+				  ETHTOOL_RING_USE_TX_PUSH,
+#endif
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 6, 0)
 	.get_link_ksettings	= ena_get_link_ksettings,
 #else
