@@ -36,6 +36,8 @@ Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
 #ifndef _KCOMPAT_H_
 #define _KCOMPAT_H_
 
+#include "config.h"
+
 #ifndef LINUX_VERSION_CODE
 #include <linux/version.h>
 #endif
@@ -94,10 +96,6 @@ Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
 #define SZ_16K 0x00004000
 #endif
 
-#ifdef HAVE_POLL_CONTROLLER
-#define CONFIG_NET_POLL_CONTROLLER
-#endif
-
 #ifndef __GFP_COLD
 #define __GFP_COLD 0
 #endif
@@ -107,6 +105,33 @@ Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
 	LINUX_VERSION_CODE >= KERNEL_VERSION(3,10,0)
 #define ENA_BUSY_POLL_SUPPORT
 #endif
+
+/* Distribution kernel version comparison macros.
+ * Distribution kernel versioning format may be A.B.C-D.E.F and standard
+ * KERNEL_VERSION macro covers only the first 3 subversions.
+ * Using 20bit per subversion, as in some cases, subversion D may be a large
+ * number (6 digits).
+ */
+#define ENA_KERNEL_VERSION_16BIT(SV1, SV2, SV3) ((SV1 << 40) | (SV2 << 20) | (SV3))
+#define ENA_KERNEL_VERSION_MAJOR(SV1, SV2, SV3) ENA_KERNEL_VERSION_16BIT(SV1, SV2, SV3)
+#define ENA_KERNEL_VERSION_MINOR(SV1, SV2, SV3) ENA_KERNEL_VERSION_16BIT(SV1, SV2, SV3)
+
+#define ENA_KERNEL_VERSION_GTE(SV1, SV2, SV3, SV4, SV5, SV6) \
+	((ENA_KERNEL_VERSION_MAJOR(ENA_KERNEL_SUBVERSION_1, ENA_KERNEL_SUBVERSION_2, ENA_KERNEL_SUBVERSION_3) > \
+	  ENA_KERNEL_VERSION_MAJOR((SV1), (SV2), (SV3))) || \
+	 (ENA_KERNEL_VERSION_MAJOR(ENA_KERNEL_SUBVERSION_1, ENA_KERNEL_SUBVERSION_2, ENA_KERNEL_SUBVERSION_3) == \
+	  ENA_KERNEL_VERSION_MAJOR((SV1), (SV2), (SV3)) && \
+	  ENA_KERNEL_VERSION_MINOR(ENA_KERNEL_SUBVERSION_4, ENA_KERNEL_SUBVERSION_5, ENA_KERNEL_SUBVERSION_6) >= \
+	  ENA_KERNEL_VERSION_MINOR((SV4), (SV5), (SV6))))
+
+#define ENA_KERNEL_VERSION_LTE(SV1, SV2, SV3, SV4, SV5, SV6) \
+	((ENA_KERNEL_VERSION_MAJOR(ENA_KERNEL_SUBVERSION_1, ENA_KERNEL_SUBVERSION_2, ENA_KERNEL_SUBVERSION_3) < \
+	  ENA_KERNEL_VERSION_MAJOR((SV1), (SV2), (SV3))) || \
+	 (ENA_KERNEL_VERSION_MAJOR(ENA_KERNEL_SUBVERSION_1, ENA_KERNEL_SUBVERSION_2, ENA_KERNEL_SUBVERSION_3) == \
+	  ENA_KERNEL_VERSION_MAJOR((SV1), (SV2), (SV3)) && \
+	  ENA_KERNEL_VERSION_MINOR(ENA_KERNEL_SUBVERSION_4, ENA_KERNEL_SUBVERSION_5, ENA_KERNEL_SUBVERSION_6) <= \
+	  ENA_KERNEL_VERSION_MINOR((SV4), (SV5), (SV6))))
+
 /******************************************************************************/
 /************************** Ubuntu macros *************************************/
 /******************************************************************************/
@@ -177,32 +202,6 @@ Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497
 #define SUSE_VERSION 0
 #endif /* SUSE_VERSION */
 
-/* SUSE kernel version comparison macros.
- * SUSE kernel versioning format may be A.B.C-D.E.F and standard KERNEL_VERSION
- * macro covers only the first 3 subversions.
- * Using 20bit per subversion, as in some cases, subversion D may be a large
- * number (6 digits).
- */
-#define SUSE_KERNEL_VERSION_16BIT(SV1, SV2, SV3) ((SV1 << 40) | (SV2 << 20) | (SV3))
-#define SUSE_KERNEL_VERSION_MAJOR(SV1, SV2, SV3) SUSE_KERNEL_VERSION_16BIT(SV1, SV2, SV3)
-#define SUSE_KERNEL_VERSION_MINOR(SV1, SV2, SV3) SUSE_KERNEL_VERSION_16BIT(SV1, SV2, SV3)
-
-#define SUSE_KERNEL_VERSION_GTE(SV1, SV2, SV3, SV4, SV5, SV6) \
-	((SUSE_KERNEL_VERSION_MAJOR(SUSE_KERNEL_SUBVERSION_1, SUSE_KERNEL_SUBVERSION_2, SUSE_KERNEL_SUBVERSION_3) > \
-	  SUSE_KERNEL_VERSION_MAJOR((SV1), (SV2), (SV3))) || \
-	 (SUSE_KERNEL_VERSION_MAJOR(SUSE_KERNEL_SUBVERSION_1, SUSE_KERNEL_SUBVERSION_2, SUSE_KERNEL_SUBVERSION_3) == \
-	  SUSE_KERNEL_VERSION_MAJOR((SV1), (SV2), (SV3)) && \
-	  SUSE_KERNEL_VERSION_MINOR(SUSE_KERNEL_SUBVERSION_4, SUSE_KERNEL_SUBVERSION_5, SUSE_KERNEL_SUBVERSION_6) >= \
-	  SUSE_KERNEL_VERSION_MINOR((SV4), (SV5), (SV6))))
-
-#define SUSE_KERNEL_VERSION_LTE(SV1, SV2, SV3, SV4, SV5, SV6) \
-	((SUSE_KERNEL_VERSION_MAJOR(SUSE_KERNEL_SUBVERSION_1, SUSE_KERNEL_SUBVERSION_2, SUSE_KERNEL_SUBVERSION_3) < \
-	  SUSE_KERNEL_VERSION_MAJOR((SV1), (SV2), (SV3))) || \
-	 (SUSE_KERNEL_VERSION_MAJOR(SUSE_KERNEL_SUBVERSION_1, SUSE_KERNEL_SUBVERSION_2, SUSE_KERNEL_SUBVERSION_3) == \
-	  SUSE_KERNEL_VERSION_MAJOR((SV1), (SV2), (SV3)) && \
-	  SUSE_KERNEL_VERSION_MINOR(SUSE_KERNEL_SUBVERSION_4, SUSE_KERNEL_SUBVERSION_5, SUSE_KERNEL_SUBVERSION_6) <= \
-	  SUSE_KERNEL_VERSION_MINOR((SV4), (SV5), (SV6))))
-
 /******************************************************************************/
 /**************************** RHEL macros *************************************/
 /******************************************************************************/
@@ -532,7 +531,8 @@ static inline unsigned int u64_stats_fetch_begin_irq(const struct u64_stats_sync
 static inline bool ena_u64_stats_fetch_retry(const struct u64_stats_sync *syncp,
 					     unsigned int start)
 {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 1, 0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 1, 0) && \
+    !(RHEL_RELEASE_CODE && RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(9,3))
 	return u64_stats_fetch_retry_irq(syncp, start);
 #else
 	return u64_stats_fetch_retry(syncp, start);
@@ -541,7 +541,8 @@ static inline bool ena_u64_stats_fetch_retry(const struct u64_stats_sync *syncp,
 
 static inline unsigned int ena_u64_stats_fetch_begin(const struct u64_stats_sync *syncp)
 {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 1, 0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 1, 0) && \
+    !(RHEL_RELEASE_CODE && RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(9,3))
 	return u64_stats_fetch_begin_irq(syncp);
 #else
 	return u64_stats_fetch_begin(syncp);
@@ -731,8 +732,8 @@ do {									\
 #define XDP_CONVERT_TO_FRAME_NAME_CHANGED
 #endif
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 9, 0)
-#define ENA_XDP_QUERY_IN_KERNEL
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 9, 0)
+#define ENA_XDP_QUERY_IN_DRIVER
 #endif
 
 #endif
@@ -744,63 +745,6 @@ do {									\
 #define HAVE_NDO_TX_TIMEOUT_STUCK_QUEUE_PARAMETER
 #endif
 
-#if defined(CONFIG_NET_DEVLINK) && LINUX_VERSION_CODE >= KERNEL_VERSION(4, 19, 0)
-#define ENA_DEVLINK_SUPPORT
-#endif
-
-#if !defined(CONFIG_NET_DEVLINK) && !defined(CONFIG_NET_DEVLINK_MODULE) && !defined(CONFIG_MAY_USE_DEVLINK)
-#define ENA_NO_DEVLINK_HEADERS
-#endif
-
-#if defined(CONFIG_NET_DEVLINK) && \
-	(KERNEL_VERSION(5, 1, 0) <= LINUX_VERSION_CODE && LINUX_VERSION_CODE < KERNEL_VERSION(5, 16, 0) && \
-	!((SUSE_VERSION != 0) && (SUSE_VERSION == 15 && (SUSE_PATCHLEVEL < 2 || SUSE_PATCHLEVEL >= 4))) && \
-	!(defined(UBUNTU_VERSION_CODE) && UBUNTU_VERSION_CODE > UBUNTU_VERSION(5, 16, 0, 0)) && \
-	!(RHEL_RELEASE_CODE))
-#define ENA_DEVLINK_PUBLISH_REQUIRED
-#endif
-
-#if defined(CONFIG_NET_DEVLINK) &&					\
-	(LINUX_VERSION_CODE >= KERNEL_VERSION(5, 4, 0) ||		\
-	 (SUSE_VERSION && LINUX_VERSION_CODE >= KERNEL_VERSION(5, 3, 18)))
-#define ENA_DEVLINK_RELOAD_UP_DOWN_SUPPORTED
-#endif
-
-#if defined(CONFIG_NET_DEVLINK) && \
-	(KERNEL_VERSION(5, 4, 0) <= LINUX_VERSION_CODE && LINUX_VERSION_CODE < KERNEL_VERSION(5, 16, 0) && \
-	!(defined(SUSE_VERSION) && (SUSE_VERSION == 15 && SUSE_PATCHLEVEL >= 4)) && \
-	!(defined(RHEL_RELEASE_CODE) && RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(9, 0)))
-#define ENA_DEVLINK_RELOAD_ENABLING_REQUIRED
-#endif
-
-#if defined(CONFIG_NET_DEVLINK) &&					\
-	(LINUX_VERSION_CODE >= KERNEL_VERSION(5, 5, 0) ||		\
-	 (SUSE_VERSION && LINUX_VERSION_CODE >= KERNEL_VERSION(5, 3, 18)))
-#define ENA_DEVLINK_RELOAD_NS_CHANGE_SUPPORT
-#endif
-
-#if defined(CONFIG_NET_DEVLINK) && LINUX_VERSION_CODE >= KERNEL_VERSION(5, 10, 0)
-#define ENA_DEVLINK_RELOAD_LIMIT_AND_ACTION_SUPPORT
-#endif
-
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 15, 0) || \
-	(defined(SUSE_VERSION) && (SUSE_VERSION == 15 && SUSE_PATCHLEVEL >= 4)) || \
-	(defined(RHEL_RELEASE_CODE) && RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(9, 0))
-#define ENA_DEVLINK_RECEIVES_DEVICE_ON_ALLOC
-#endif
-
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 16, 0) || \
-	(defined(SUSE_VERSION) && (SUSE_VERSION == 15 && SUSE_PATCHLEVEL >= 4))  || \
-	(defined(RHEL_RELEASE_CODE) && RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(9, 0))
-#define ENA_DEVLINK_RELOAD_SUPPORT_ADVERTISEMENT_NEEDED
-#endif
-
-#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 16, 0) && \
-	!(defined(SUSE_VERSION) && (SUSE_VERSION == 15 && SUSE_PATCHLEVEL >= 4)) && \
-	!(defined(RHEL_RELEASE_CODE) && RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(9, 0))
-#define ENA_DEVLINK_CONFIGURE_AFTER_REGISTER
-#endif
-
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3,19,0) && \
     !(RHEL_RELEASE_CODE && ((RHEL_RELEASE_CODE != RHEL_RELEASE_VERSION(7, 1)) && \
 			    (RHEL_RELEASE_CODE > RHEL_RELEASE_VERSION(6, 6)))) && \
@@ -847,17 +791,32 @@ static inline void netdev_rss_key_fill(void *buffer, size_t len)
 
 #if ((LINUX_VERSION_CODE < KERNEL_VERSION(4, 6 ,0)) && \
      !(RHEL_RELEASE_CODE && RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(7,3)))
-/* Linux versions 4.4.216 - 4.5 (non inclusive) back propagated page_ref_count
- * function from kernel 4.6. To make things more difficult, Ubuntu didn't add
- * these changes to its 4.4.* kernels
+/* Linux versions 4.4.216 - 4.5 (non inclusive) back propagated
+ * page_ref_count() from kernel 4.6.
+ * Ubuntu didn't add these changes to its 4.4.* kernels.
+ * UEK added this function in kernel 4.1.12-124.43.1
+ * Here is a figure that shows all of the cases:
+ * Legend:
+ * -------- page_ref_count() is present in the kernel
+ * ******** page_ref_count() is missing in the kernel
+ *
+ * Distro\Kernel      4.1.12-124.43.1   4.4.216    4.5    4.6
+ *                           |              |        |      |
+ * Upstrem kernel ***********|**************|--------|******|
+ *                           |              |        |      |
+ * Ubuntu         ***********|**************|********|******|
+ *                           |              |        |      |
+ * UEK            ***********|--------------|--------|------|
  */
-#if !(KERNEL_VERSION(4, 4 ,216) <= LINUX_VERSION_CODE && LINUX_VERSION_CODE < KERNEL_VERSION(4, 5 ,0)) ||\
-      defined(UBUNTU_VERSION_CODE)
+#if (defined(IS_UEK) && !ENA_KERNEL_VERSION_GTE(4, 1, 12, 124, 43, 1)) || \
+    (defined(UBUNTU_VERSION_CODE)) || \
+    (!defined(IS_UEK) && !defined(UBUNTU_VERSION_CODE) && \
+     !(KERNEL_VERSION(4, 4, 216) <= LINUX_VERSION_CODE && LINUX_VERSION_CODE < KERNEL_VERSION(4, 5, 0)))
 static inline int page_ref_count(struct page *page)
 {
 	return atomic_read(&page->_count);
 }
-#endif /* !(KERNEL_VERSION(4, 4 ,216) <= LINUX_VERSION_CODE && LINUX_VERSION_CODE < KERNEL_VERSION(4, 5 ,0)) */
+#endif /* (defined(IS_UEK) && !ENA_KERNEL_VERSION_GTE(4, 1, 12, 124, 43, 1)) ... */
 
 static inline void page_ref_inc(struct page *page)
 {
@@ -915,7 +874,7 @@ static inline int numa_mem_id(void)
 #if defined(ENA_XDP_SUPPORT) && \
 	(LINUX_VERSION_CODE < KERNEL_VERSION(5, 12, 0) && \
 	!(defined(SUSE_VERSION) && (SUSE_VERSION == 15 && SUSE_PATCHLEVEL == 3) && \
-	 SUSE_KERNEL_VERSION_GTE(5, 3, 18, 150300, 59, 49)))
+	 ENA_KERNEL_VERSION_GTE(5, 3, 18, 150300, 59, 49)))
 static __always_inline void
 xdp_init_buff(struct xdp_buff *xdp, u32 frame_sz, struct xdp_rxq_info *rxq)
 {
@@ -951,11 +910,15 @@ xdp_prepare_buff(struct xdp_buff *xdp, unsigned char *hard_start,
 #define ENA_XDP_XMIT_FREES_FAILED_DESCS_INTERNALLY
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 15, 0) && \
+#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 15, 0) && \
+	!(LINUX_VERSION_CODE >= KERNEL_VERSION(5, 10, 188) && \
+	 LINUX_VERSION_CODE < KERNEL_VERSION(5, 11, 0)) && \
+	!(LINUX_VERSION_CODE >= KERNEL_VERSION(5, 4, 251) && \
+	 LINUX_VERSION_CODE < KERNEL_VERSION(5, 5, 0))) && \
 	!(defined(RHEL_RELEASE_CODE) && RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(8, 6)) && \
 	!(defined(SUSE_VERSION) && (SUSE_VERSION == 15 && SUSE_PATCHLEVEL >= 4)) && \
 	!(defined(SUSE_VERSION) && (SUSE_VERSION == 15 && SUSE_PATCHLEVEL == 3) && \
-	  SUSE_KERNEL_VERSION_GTE(5, 3, 18, 150300, 59, 43))
+	  ENA_KERNEL_VERSION_GTE(5, 3, 18, 150300, 59, 43))
 static inline void eth_hw_addr_set(struct net_device *dev, const u8 *addr)
 {
 	memcpy(dev->dev_addr, addr, ETH_ALEN);
@@ -973,7 +936,8 @@ static inline void eth_hw_addr_set(struct net_device *dev, const u8 *addr)
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 17, 0) || \
 	(defined(RHEL_RELEASE_CODE) && \
 	RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(8, 7) && \
-	RHEL_RELEASE_CODE != RHEL_RELEASE_VERSION(9, 0))
+	RHEL_RELEASE_CODE != RHEL_RELEASE_VERSION(9, 0)) || \
+	(defined(SUSE_VERSION) && (SUSE_VERSION == 15 && SUSE_PATCHLEVEL >= 5))
 #define ENA_ETHTOOL_RX_BUFF_SIZE_CHANGE
 #endif
 
@@ -995,7 +959,8 @@ static inline int netif_xmit_stopped(const struct netdev_queue *dev_queue)
 #define NAPIF_STATE_SCHED BIT(NAPI_STATE_SCHED)
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 17, 0)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 17, 0) && \
+	!(defined(IS_UEK) && ENA_KERNEL_VERSION_GTE(5, 15, 0, 100, 96, 32))
 #define bpf_warn_invalid_xdp_action(netdev, xdp_prog, verdict) \
 	bpf_warn_invalid_xdp_action(verdict)
 #endif
@@ -1046,6 +1011,10 @@ static inline bool ktime_after(const ktime_t cmp1, const ktime_t cmp2)
 #define ENA_PHC_SUPPORT_GETTIME64_EXTENDED
 #endif /* ENA_PHC_SUPPORT_GETTIME64_EXTENDED */
 
+#if ((LINUX_VERSION_CODE >= KERNEL_VERSION(3, 0, 0)) && (LINUX_VERSION_CODE < KERNEL_VERSION(6, 2, 0)))
+#define ENA_PHC_SUPPORT_ADJFREQ
+#endif /* ENA_PHC_SUPPORT_ADJFREQ */
+
 #if ((LINUX_VERSION_CODE < KERNEL_VERSION(3, 7, 0)) && \
 	!(RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(6, 4))))
 #define ptp_clock_register(info, parent) ptp_clock_register(info)
@@ -1076,7 +1045,11 @@ static inline void ena_netif_napi_add(struct net_device *dev,
 				      struct napi_struct *napi,
 				      int (*poll)(struct napi_struct *, int))
 {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 1, 0)
+#if (LINUX_VERSION_CODE < KERNEL_VERSION(6, 1, 0)) && \
+	!(RHEL_RELEASE_CODE && \
+	  ((RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(8, 8)) && \
+	   (RHEL_RELEASE_CODE < RHEL_RELEASE_VERSION(9, 0))) || \
+	  (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(9, 2)))
 #ifndef NAPI_POLL_WEIGHT
 #define NAPI_POLL_WEIGHT 64
 #endif
@@ -1086,4 +1059,80 @@ static inline void ena_netif_napi_add(struct net_device *dev,
 #endif /* LINUX_VERSION_CODE < KERNEL_VERSION(6, 1, 0) */
 }
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 4, 0)
+#define ENA_LARGE_LLQ_ETHTOOL
+#endif
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 9, 0)
+#include <linux/bitfield.h>
+#define ENA_FIELD_GET(value, mask, offset) FIELD_GET(mask, value)
+#else
+#define ENA_FIELD_GET(value, mask, offset) ((typeof(mask))((value & mask) >> offset))
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 3, 0)
+#define xdp_features_set_redirect_target(netdev, xdp_xmit_supported)
+#define xdp_features_clear_redirect_target(netdev)
+#define xdp_clear_features_flag(netdev)
+#define xdp_set_features_flag(netdev, features)
+#else /* LINUX_VERSION_CODE < KERNEL_VERSION(6, 3, 0) */
+#define ENA_XDP_NETLINK_ADVERTISEMENT
+#endif
+
+#if (RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE <= RHEL_RELEASE_VERSION(7, 4))) || \
+    (defined(UBUNTU_VERSION_CODE) && UBUNTU_VERSION_CODE < UBUNTU_VERSION(4, 5, 0, 0)) || \
+    (defined(IS_UEK) && !ENA_KERNEL_VERSION_GTE(4, 1, 12, 105, 0, 0))
+static inline void dma_unmap_page_attrs(struct device *dev,
+					dma_addr_t addr, size_t size,
+					enum dma_data_direction dir,
+					struct dma_attrs *attrs)
+{
+	struct dma_map_ops *ops = get_dma_ops(dev);
+
+	BUG_ON(!valid_dma_direction(dir));
+	if (ops->unmap_page)
+		ops->unmap_page(dev, addr, size, dir, attrs);
+	debug_dma_unmap_page(dev, addr, size, dir, false);
+}
+#endif /* RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE <= RHEL_RELEASE_VERSION(7, 4)) */
+
+#if (RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE <= RHEL_RELEASE_VERSION(7, 9)) && \
+     (RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(7, 0)) && \
+     (LINUX_VERSION_CODE != KERNEL_VERSION(4, 14, 0))) || \
+    (defined(UBUNTU_VERSION_CODE) && UBUNTU_VERSION_CODE < UBUNTU_VERSION(4, 5, 0, 0)) || \
+    (defined(IS_UEK) && LINUX_VERSION_CODE < KERNEL_VERSION(4, 1, 13))
+#define ENA_DMA_ATTR_SKIP_CPU_SYNC (1 << DMA_ATTR_SKIP_CPU_SYNC)
+#elif (RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE <= RHEL_RELEASE_VERSION(6, 10)))
+#define	ENA_DMA_ATTR_SKIP_CPU_SYNC 0
+#else
+#define ENA_DMA_ATTR_SKIP_CPU_SYNC DMA_ATTR_SKIP_CPU_SYNC
+#endif
+
+static inline void ena_dma_unmap_page_attrs(struct device *dev,
+					    dma_addr_t addr, size_t size,
+					    enum dma_data_direction dir,
+					    unsigned long attrs)
+{
+#if (RHEL_RELEASE_CODE && (RHEL_RELEASE_CODE <= RHEL_RELEASE_VERSION(7, 9)) && \
+     (LINUX_VERSION_CODE != KERNEL_VERSION(4, 14, 0))) || \
+    (defined(UBUNTU_VERSION_CODE) && UBUNTU_VERSION_CODE < UBUNTU_VERSION(4, 5, 0, 0)) || \
+    (defined(IS_UEK) && LINUX_VERSION_CODE < KERNEL_VERSION(4, 1, 13))
+	struct dma_attrs dma_attrs;
+
+	init_dma_attrs(&dma_attrs);
+	dma_attrs.flags[0] = attrs;
+	dma_unmap_page_attrs(dev, addr, size, dir, &dma_attrs);
+#else
+	dma_unmap_page_attrs(dev, addr, size, dir, attrs);
+#endif
+}
+
+#ifndef ENA_HAVE_PCI_DEV_ID
+#define pci_dev_id(pdev) ((((u16)(pdev->bus->number)) << 8) | (pdev->devfn))
+#endif /* ENA_HAVE_PCI_DEV_ID */
+
+#ifndef ENA_HAVE_XDP_DO_FLUSH
+#define xdp_do_flush xdp_do_flush_map
+#endif /* ENA_HAVE_XDP_DO_FLUSH */
+
 #endif /* _KCOMPAT_H_ */
